searchData={"items":[{"type":"task","title":"mix deps.smee","doc":"Install backend CLI commands required by Smee","ref":"Mix.Tasks.Deps.Smee.html"},{"type":"module","title":"Smee","doc":"`Smee` is a pragmatic library for handling SAML metadata with Elixir, Erlang or any other BEAM language.","ref":"Smee.html"},{"type":"module","title":"Features - Smee","doc":"* Download remote SAML metadata or load local files, with effective caching\n* Manage and compare metadata files and individual entity metadata\n* MDQ API (which can also emulate MDQ style lookups with aggregate files)\n* A focus on streaming with reliable and surprisingly low memory usage\n* Filter entity streams by various criteria\n* Validate XML signatures, automatically download and confirm signing certificates\n* Transform metadata using XSLT, or extract data\n* Access XML using Erlang's Xmerl library (sweetened by SweetXML)\n* Recombine entity streams into aggregates or other data formats\n* Can be used with applications or in simple .exs scripts","ref":"Smee.html#module-features"},{"type":"module","title":"Modules - Smee","doc":"The top level `Smee` module contains a few simplified, top level functions better suited to simpler scripts. Other modules in\nSmee contain more tools for handling SAML metadata, such as:\n\n* `Smee.Source` - define sources of metadata\n* `Smee.Metadata` - functions for handling metadata aggregates\n* `Smee.Entity` - individual SAML entity definitions\n* `Smee.Extract` - processing metadata to extract information\n* `Smee.Fetch` - downloading metadata sources\n* `Smee.MDQ` - functions for MDQ clients (and emulating MDQ clients)\n* `Smee.Filter` - filtering streams of entity records\n* `Smee.Transform` - processing and editing entity XML\n* `Smee.Publish` - Formatting and outputting metadata in various formats\n* `Smee.Stats` - Simple stats for entity streams\n* `Smee.Lint` - XML validation and reformatting","ref":"Smee.html#module-modules"},{"type":"function","title":"Smee.entity_ids/1","doc":"Lists the IDs of every entity in the metadata.\n\nThis version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.","ref":"Smee.html#entity_ids/1"},{"type":"function","title":"Example - Smee.entity_ids/1","doc":"iex> Smee.source(\"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\")\n    iex> |> Smee.entity_ids()","ref":"Smee.html#entity_ids/1-example"},{"type":"function","title":"Smee.fetch!/1","doc":"Downloads a source of metadata (local or remote) and returns a %Metadata{} struct containing XML and information.","ref":"Smee.html#fetch!/1"},{"type":"function","title":"Example - Smee.fetch!/1","doc":"iex> \"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\"\n    iex> |> Smee.source()\n    iex> |> Smee.fetch!()","ref":"Smee.html#fetch!/1-example"},{"type":"function","title":"Smee.lookup!/2","doc":"Retrieves information for a single entity from an MDQ service (real or emulated) and returns an %Entity{} struct.\n\nThis version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.","ref":"Smee.html#lookup!/2"},{"type":"function","title":"Example - Smee.lookup!/2","doc":"iex> \"http://mdq.ukfederation.org.uk/\"\n    iex> |> Smee.source(type: :mdq)\n    iex> |> Smee.lookup!(\"https://cern.ch/login\")\n\n    iex> \"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\"\n    iex> |> Smee.source(type: :aggregate)\n    iex> |> Smee.lookup!(\"https://cern.ch/login\")","ref":"Smee.html#lookup!/2-example"},{"type":"function","title":"Smee.source/1","doc":"Defines a source of metadata\n\n  Sources of metadata include online aggregate XML, local aggregate files, individual entities, and MDQ services.\n  This function will only define sources of aggregate XML.","ref":"Smee.html#source/1"},{"type":"function","title":"Example - Smee.source/1","doc":"iex> Smee.source(\"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\")\n    iex>","ref":"Smee.html#source/1-example"},{"type":"function","title":"Smee.source/2","doc":"Defines a source of metadata\n\n Sources of metadata include online aggregate XML, local aggregate files, individual entities, and MDQ services.\n This function allows a lot of customisation, particularly the *type*. Types are:\n * :aggregate (a file containing a collection of\n entityDescriptor fragments inside a entitiesDescriptor tag, as used by federations)\n * :single (a file with a single entityDescriptor,\n as used for individual metadata records)\n *  :mdq (an online MDQ service)\n\nURLs may be remote (http:// and https://) or local (file://). Local files can be specified as bare paths.\n\nSee `Smee.Source.new` for full details","ref":"Smee.html#source/2"},{"type":"function","title":"Example - Smee.source/2","doc":"iex> Smee.source(\"http://mdq.ukfederation.org.uk/\", type: :mdq, retries: 1, label: \"UK MDQ Service\")\n    iex> Smee.source(\"support/static/valid.xml\", type: :single, retries: 1, label: \"My IdP\")","ref":"Smee.html#source/2-example"},{"type":"function","title":"Smee.stream_entities/1","doc":"Streams all entities in the specified metadata or source.\n\nThis version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.","ref":"Smee.html#stream_entities/1"},{"type":"function","title":"Example - Smee.stream_entities/1","doc":"iex> Smee.stream_entities(\"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\")\n    iex> |> Stream.take(1)\n    iex> |> Enum.to_list","ref":"Smee.html#stream_entities/1-example"},{"type":"module","title":"Smee.Entity","doc":"`Smee` wraps up metadata for individual entities in %Entity{} structs, and the `Smee.Entity` module contains\n  functions that may be useful when working with them.\n\nMany of the functions mirror those in the `Smee.Metadata` module - the same actions but on a smaller unit of XML.\n\nLike %Metadata{} structs the XML in entities can be compressed and decompressed, but unlike Metadata structs they have\n  parsed xmerl data in them by default too.\n\nWherever possible use `Entity.update/2` to make changes, do not write to the Entity struct directly. If you must write directly\n  you can use `Entity.update/1` to resync the state of the record.","ref":"Smee.Entity.html"},{"type":"function","title":"Smee.Entity.assurance/1","doc":"Returns the assurance certification URIs of the entity as a list\n\nThe most common and useful assurance profile is almost certainly SIRTFI, but many others exist.","ref":"Smee.Entity.html#assurance/1"},{"type":"function","title":"Smee.Entity.bulkup/1","doc":"Returns an entity that contains parsed XML data, greatly increasing its size but possibly making future processing faster.","ref":"Smee.Entity.html#bulkup/1"},{"type":"function","title":"Smee.Entity.categories/1","doc":"Returns a list of entity categories for this entity\n\nEntity categories are official tags indicating that a service fits certain criteria. For instance, Research and Scholarship\n(indicated with the URI \"http://refeds.org/category/research-and-scholarship\") shows that a service will behave according to\n  certain rules.\n\nYou can see a dump of all entity attributes, including entity categories, using `Smee.Extract.list_entity_attrs/1`","ref":"Smee.Entity.html#categories/1"},{"type":"function","title":"Smee.Entity.category_support/1","doc":"Returns a list of *supported* entity categories\n\nCategory support is the counterpart of entity categories, indicating that a service will make use of those categories if\n  an entity possesses them.","ref":"Smee.Entity.html#category_support/1"},{"type":"function","title":"Smee.Entity.check_date!/1","doc":"Raises an exception if the entity has expired (based on valid_until datetime), otherwise returns the entity.\n\nIf no valid_until has been set (if it's nil) then the entity will always be returned.","ref":"Smee.Entity.html#check_date!/1"},{"type":"function","title":"Smee.Entity.compress/1","doc":"Returns a compressed entity, containing gzipped XML. This greatly reduces the size of the entity record.","ref":"Smee.Entity.html#compress/1"},{"type":"function","title":"Smee.Entity.compressed?/1","doc":"Returns true if the XML data in an entity has been compressed","ref":"Smee.Entity.html#compressed?/1"},{"type":"function","title":"Smee.Entity.decompress/1","doc":"Returns a decompressed entity, with plain-text XML data. This makes the struct much larger.","ref":"Smee.Entity.html#decompress/1"},{"type":"function","title":"Smee.Entity.derive/3","doc":"Returns a new %Entity{} struct if passed XML data for an entity *and* parent %Metadata{}.\n\nDefaults values are set using the parent metadata where possible.\n\nYou can set or override various parts of the struct by passing options:\n\n* md_uri - a URI that identifies a parent\n* downloaded_at - A DateTime to record when the record was downloaded\n* modified_at - A DateTime to record when the record was updated *upstream*\n* valid_until - A DateTime to indicate when an entity expires\n* priority - An integer between 0 and 9 to show priority\n* trustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.\n\nYou won't normally need to do this yourself as entities can be extracted from `Smee.Metadata`.","ref":"Smee.Entity.html#derive/3"},{"type":"function","title":"Smee.Entity.expired?/1","doc":"Returns true if the entity has expired (based on valid_until datetime)\n\nIf no valid_until has been set (if it's nil) then false will be returned","ref":"Smee.Entity.html#expired?/1"},{"type":"function","title":"Smee.Entity.filename/1","doc":"Returns a suggested filename for the entity.","ref":"Smee.Entity.html#filename/1"},{"type":"function","title":"Smee.Entity.filename/2","doc":"Returns a suggested filename for the entity in the specified format.\n\nTwo formats can be specified: :sha1 and :uri","ref":"Smee.Entity.html#filename/2"},{"type":"function","title":"Smee.Entity.id/1","doc":"Returns the entityID of the entity struct.\n\n(This is the same as the URI)","ref":"Smee.Entity.html#id/1"},{"type":"function","title":"Smee.Entity.idp?/1","doc":"Returns true if the entity has an IdP role.\n\nAn entity may have more than one role.","ref":"Smee.Entity.html#idp?/1"},{"type":"function","title":"Smee.Entity.new/2","doc":"Returns a new %Entity{} struct if passed XML data.\n\nYou can set or override various parts of the struct by passing options:\n\n* md_uri - a URI that identifies a parent\n* downloaded_at - A DateTime to record when the record was downloaded\n* modified_at - A DateTime to record when the record was updated *upstream*\n* valid_until - A DateTime to indicate when an entity expires\n* priority - An integer between 0 and 9 to show priority\n* trustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.\n\nYou won't normally need to do this yourself as entities can be extracted from `Smee.Metadata`.","ref":"Smee.Entity.html#new/2"},{"type":"function","title":"Smee.Entity.priority/1","doc":"Returns the priority of the entity as a value between 0 and 9","ref":"Smee.Entity.html#priority/1"},{"type":"function","title":"Smee.Entity.registered_at/1","doc":"Returns the Date of registration, if present, or nil.\n\nSee `registration_authority/1` for more information","ref":"Smee.Entity.html#registered_at/1"},{"type":"function","title":"Smee.Entity.registration_authority/1","doc":"Returns the URI for the entity's registration authority, if available, or nil.\n\nInterfederation means that entities may be initially published in a different federation to the one *this* record is\n  currently published in. The registration authority URI, if present, shows that the entity was officially registered\nelsewhere.","ref":"Smee.Entity.html#registration_authority/1"},{"type":"function","title":"Smee.Entity.slim/1","doc":"Returns an entity with parsed XML data removed, greatly reducing its size but possibly making future processing slower.","ref":"Smee.Entity.html#slim/1"},{"type":"function","title":"Smee.Entity.sp?/1","doc":"Returns true if the entity has an SP role.\n\nAn entity may have more than one role.","ref":"Smee.Entity.html#sp?/1"},{"type":"function","title":"Smee.Entity.tag/2","doc":"Tags an entity with one or more tags, replacing existing tags.\n\nTags are arbitrary classifiers, initially inherited from source and metadata","ref":"Smee.Entity.html#tag/2"},{"type":"function","title":"Smee.Entity.tags/1","doc":"Returns the tags of the entity struct, a list of binary strings\n\nTags are arbitrary strings, which may be initially inherited from source and metadata","ref":"Smee.Entity.html#tags/1"},{"type":"function","title":"Smee.Entity.transformed_id/1","doc":"Returns the MDQ hashed style entityID of the entity struct.\n\n(This is the same as the URI)","ref":"Smee.Entity.html#transformed_id/1"},{"type":"function","title":"Smee.Entity.trustiness/1","doc":"Returns the trustiness level of the entity as a float between 0.0 and 0.9.","ref":"Smee.Entity.html#trustiness/1"},{"type":"function","title":"Smee.Entity.update/1","doc":"Resyncs the internal state of an %Entity{} struct\n\nIf changes have been made using `Entity.update/2` then this will not be needed - it's there for when the struct\n  has been changed directly","ref":"Smee.Entity.html#update/1"},{"type":"function","title":"Smee.Entity.update/2","doc":"Returns an updates %Entity{} struct with new XML, refreshing various parts of the struct correctly.\n\nThis should be the only way updated Entities are produced - the raw struct should not be changed directly.","ref":"Smee.Entity.html#update/2"},{"type":"function","title":"Smee.Entity.validate!/1","doc":"Raises an exception if the entity has invalid XML, otherwise returns the entity.","ref":"Smee.Entity.html#validate!/1"},{"type":"function","title":"Smee.Entity.xdoc/1","doc":"Returns a parsed Erlang `xmerl` structure representing the entities XML, for use with `xmerl`, `SweetXML` and other\n  tools.","ref":"Smee.Entity.html#xdoc/1"},{"type":"function","title":"Smee.Entity.xml/1","doc":"Returns the plain-text XML of the entity, whether or not it has been compressed.","ref":"Smee.Entity.html#xml/1"},{"type":"type","title":"Smee.Entity.t/0","doc":"","ref":"Smee.Entity.html#t:t/0"},{"type":"module","title":"Smee.Extract","doc":"Processes `%Metadata{}` structs to extract various information, usually using XSLT.\n\nThis module is intended to make common and relatively simple actions on potentially large XML files more efficiently than\n  processing each %Entity{} struct in turn, and may rely on external tools such as `xsltproc`.\n\nThese functions are useful in quick reports or to speed-up the internal workings of other modules.","ref":"Smee.Extract.html"},{"type":"function","title":"Smee.Extract.entity!/2","doc":"Returns a single %Entity{} struct extracted from the metadata, if present. Raises an exception if not present.","ref":"Smee.Extract.html#entity!/2"},{"type":"function","title":"Smee.Extract.list_entity_attrs/1","doc":"Returns a map of all unique metadata Entity Attributes and their values","ref":"Smee.Extract.html#list_entity_attrs/1"},{"type":"function","title":"Smee.Extract.list_ids/1","doc":"Returns a list of all entity IDs in a Metadata struct.","ref":"Smee.Extract.html#list_ids/1"},{"type":"function","title":"Smee.Extract.mdui_info/1","doc":"Returns a list of maps containing MDUI information for each entity in the metadata file.","ref":"Smee.Extract.html#mdui_info/1"},{"type":"module","title":"Smee.Fetch","doc":"Downloads or loads the metadata specified in a `%Smee.Source{}` struct and returns a `%Smee.Metadata{}` struct.\n\nThe `fetch!/2` and `fetch/2` functions can handle both remote and local sources, but sometimes it can be\n  reassuring to prevent unexpected behaviour (perhaps if using a source defined by a user) so `Smee.Fetch` also\ncontains `local!/2` and `remote!/2` which will each only accept certain types of sources.\n\nThis module will do the work of creating suitable Metadata structs for you, so you should not normally need to\n  create Metadata structs directly yourself.","ref":"Smee.Fetch.html"},{"type":"function","title":"Smee.Fetch.fetch/2","doc":"Uses the passed Source struct to load or download the requested metadata XML, and returns a Metadata struct containing the XML\ninside an :ok tuple.\n\nWorks with all types of Source, even MDQ services.","ref":"Smee.Fetch.html#fetch/2"},{"type":"function","title":"Smee.Fetch.fetch!/2","doc":"Uses the passed Source struct to load or download the requested metadata XML, and returns a Metadata struct containing the XML.\n\nWorks with all types of Source, even MDQ services.\n\nWill raise an exception on any errors.","ref":"Smee.Fetch.html#fetch!/2"},{"type":"function","title":"Smee.Fetch.local/2","doc":"Uses the passed Source struct to load the requested metadata XML, and returns a Metadata struct containing\n the XML in an :ok tuple\n\n Works with local Sources including MDQ services but will not accept metadata at a remote URL.","ref":"Smee.Fetch.html#local/2"},{"type":"function","title":"Smee.Fetch.local!/2","doc":"Uses the passed Source struct to load the requested metadata XML, and returns a Metadata struct containing\n the XML\n\n Works with local Sources including MDQ services but will not accept metadata at a remote URL.\n\n Will raise an exception on any errors.","ref":"Smee.Fetch.html#local!/2"},{"type":"function","title":"Smee.Fetch.probe/1","doc":"Connects to a remote Source and uses HEAD HTTP method, with no caching, to fetch version/change information from headers.\n\nAn :ok tuple containing a map of etag (string) and changed_at (DateTime) will be returned if successful.","ref":"Smee.Fetch.html#probe/1"},{"type":"function","title":"Smee.Fetch.remote/2","doc":"Uses the passed Source struct to download the requested metadata XML, and returns a Metadata struct containing\n the XML in an :ok/:error tuple.\n\n   Works with remote Sources including MDQ services but will not accept metadata in a local file.","ref":"Smee.Fetch.html#remote/2"},{"type":"function","title":"Smee.Fetch.remote!/2","doc":"Uses the passed Source struct to download the requested metadata XML, and returns a Metadata struct containing\n the XML\n\n Works with remote Sources including MDQ services but will not accept metadata in a local file.\n\n Will raise an exception on any errors.","ref":"Smee.Fetch.html#remote!/2"},{"type":"function","title":"Smee.Fetch.warm/2","doc":"Downloads one or more Sources in parallel and caches the result in the HTTP cache.\n\nA map of source URLs and status codes are returned. A status code of 0 indicates a client error of some sort.\n\nThis function is useful for downloading source metadata in advance so that processing can happen immediately.","ref":"Smee.Fetch.html#warm/2"},{"type":"module","title":"Smee.Filter","doc":"Process a stream of entities to include or exclude entity structs matching the specified criteria.\n\nThese functions are intended to be used with streams but should also work with simple lists too - but using lists to\nprocess larger metadata files is **strongly discouraged**.\n\nBy default these functions include matching entities and exclude those that do not match, but this an be reversed.\nBy default `Smee.Filter.idp/2` will exclude entities that are no IdPs. But by specifying `false` as the third\nparameter the filter will be inverted and exclude entities that have an IdP role.","ref":"Smee.Filter.html"},{"type":"function","title":"Smee.Filter.assurance/3","doc":"Filters a stream of entities to include or exclude those that have the specified assurance profile certification URI.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#assurance/3"},{"type":"function","title":"Smee.Filter.days/3","doc":"Filters a stream of entities to include or exclude those that were registered within the specified number of days.\n\nEntities without a date will be included in the inverted results.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#days/3"},{"type":"function","title":"Smee.Filter.entity_category/3","doc":"Filters a stream of entities to include or exclude those that have the specified entity category URI.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#entity_category/3"},{"type":"function","title":"Smee.Filter.entity_category_support/3","doc":"Filters a stream of entities to include or exclude those that have the specified entity category support URI.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#entity_category_support/3"},{"type":"function","title":"Smee.Filter.fresh/2","doc":"Filters a stream of entities to include or exclude those that were registered within the last 7 days.\n\nEntities without a date will be included in the inverted results.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#fresh/2"},{"type":"function","title":"Smee.Filter.idp/2","doc":"Filters a stream of entities to include or exclude those that have an IdP role.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#idp/2"},{"type":"function","title":"Smee.Filter.registered_after/3","doc":"Filters a stream of entities to include or exclude those that were registered after the specified date.\n\nDates can be Date structs or binary strings in the format \"YYYY-MM-DD\"\n\nEntities without a date will be included in the inverted results.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#registered_after/3"},{"type":"function","title":"Smee.Filter.registered_before/3","doc":"Filters a stream of entities to include or exclude those that were registered before the specified date.\n\nDates can be Date structs or binary strings in the format \"YYYY-MM-DD\"\n\nEntities without a date will be included in the inverted results.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#registered_before/3"},{"type":"function","title":"Smee.Filter.registered_by/3","doc":"Filters a stream of entities to include or exclude those that were registered at a particular federation, using the\n  specified URI.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#registered_by/3"},{"type":"function","title":"Smee.Filter.sp/2","doc":"Filters a stream of entities to include or exclude those that have an SP role.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#sp/2"},{"type":"function","title":"Smee.Filter.tag/3","doc":"Filters a stream of entities to include or exclude those that have the specified tag.\n\nIt's best to provide the tag as a string.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#tag/3"},{"type":"function","title":"Smee.Filter.trustiness/3","doc":"Filters a stream of entities to include or exclude those that have a trustiness equal or less than the specified number.\n\nTrustiness values are between 0.0 and 0.9.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#trustiness/3"},{"type":"function","title":"Smee.Filter.uri/3","doc":"Filters a stream of entities to include or exclude those that have one of the specified IDs.\n\nThe filter is positive by default but can be inverted by specifying `false`","ref":"Smee.Filter.html#uri/3"},{"type":"module","title":"Smee.Lint","doc":"Lint runs basic XML quality checks against XML strings","ref":"Smee.Lint.html"},{"type":"function","title":"Smee.Lint.validate/2","doc":"Validates XML - checks that it is well-formed and complies with SAML metadata schema. It does **not* check\n  signatures or expiry.","ref":"Smee.Lint.html#validate/2"},{"type":"module","title":"Smee.MDQ","doc":"`Smee.MDQ` provides a client API to MDQ services, and also attempts to emulate MDQ-style behaviour with old-fashioned\n  metadata aggregates. MDQ allows individual entities to be looked up when needed without downloading and processing\n  an increasingly enormous metadata aggregate file.\n\nEmulation of MDQ for aggregates depends on caching and is usually slower for individual entity lookups even after the\n  initial download and caching is complete.","ref":"Smee.MDQ.html"},{"type":"function","title":"Smee.MDQ.aggregate!/1","doc":"Returns the aggregated XML for the MDQ service as a Metadata struct, if one is available.","ref":"Smee.MDQ.html#aggregate!/1"},{"type":"function","title":"Smee.MDQ.get/2","doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.\n\nGet attempts to behave like Ecto's `Repo.get` - it will return an entity or nil if the entity is unavailable.","ref":"Smee.MDQ.html#get/2"},{"type":"function","title":"Smee.MDQ.get!/2","doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.\n\nGet attempts to behave like Ecto's `Repo.get!` - it will return an entity or raises an exception if the entity is unavailable.\n\nThis is identical to `lookup!2` but exists for consistency.","ref":"Smee.MDQ.html#get!/2"},{"type":"function","title":"Smee.MDQ.list!/1","doc":"Returns a list of all entity ID URIs available at the MDQ service.\n\nThis will probably involve downloading the MDQ service's aggregate in most cases, but this aggregate will be cached.","ref":"Smee.MDQ.html#list!/1"},{"type":"function","title":"Smee.MDQ.lookup/2","doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct in an :ok/:error tuple.\n\nMissing or unknown entities will cause an {:error, :http_404} result.","ref":"Smee.MDQ.html#lookup/2"},{"type":"function","title":"Smee.MDQ.lookup!/2","doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.\n\nThis is identical to `get!2` but exists for consistency.","ref":"Smee.MDQ.html#lookup!/2"},{"type":"function","title":"Smee.MDQ.source/2","doc":"A convenient shortcut for specifying an MDQ service as a source.\n\nThis is functionally identical to using `Smee.Source.new(url, type: :mdq)`","ref":"Smee.MDQ.html#source/2"},{"type":"function","title":"Smee.MDQ.stream/1","doc":"Fetches *every entity* from the MDQ service one by one and returns them as a stream of Entity structs.\n\nThis stream allows entities to be processed individually, using relatively little memory.\n\nThe stream is relatively laid-back even on a fast M1 Mac, but please don't abuse this function and overwhelm public MDQ\n  services.","ref":"Smee.MDQ.html#stream/1"},{"type":"function","title":"Smee.MDQ.stream/2","doc":"Fetches the specified entities from the MDQ service one by one and returns them as a stream of Entity structs.\n\nThis stream allows entities to be processed individually, using relatively little memory.\n\nThe stream is relatively laid-back even on a fast M1 Mac, but please don't abuse this function and overwhelm public MDQ\n  services.","ref":"Smee.MDQ.html#stream/2"},{"type":"function","title":"Smee.MDQ.transform_uri/1","doc":"If passed an entity ID URI is returns the MDQ \"transformed\" version of the identifer, based on a sha1 hash.\n\nAlready-transformed identifiers are passed through unchanged.","ref":"Smee.MDQ.html#transform_uri/1"},{"type":"function","title":"Smee.MDQ.url/2","doc":"Returns the full download URL for an entity at the specified service.","ref":"Smee.MDQ.html#url/2"},{"type":"module","title":"Smee.Metadata","doc":"The Metadata module wraps up Metadata XML into a struct and contains functions that may be helpful when working with\n  them. The metadata is either an aggregate (as used by federations to contain many entity records) or a single entity.\n\n  Many of the functions mirror those in the `Smee.Entity` module - the same actions but on larger source XML rather\nthan on fragments.\n\nThe XML in metadata structs can be compressed or decompressed, but unlike Entities there is no cached, parsed xmlerl record\n  by default - this is to save time and memory.\n\n   Wherever possible use `Metadata.update/2` to make changes, do not write to the Entity struct directly. If you must write directly\n  you can use `Metadata.update/1` to resync the state of the record.\n\nMethods in `Smee.Metadata` can be used to extract individual entity records each containing a fragment of XML. It's\n  strongly recommended to stream these using `stream_entities2` to save on memory, or select a particular entity using\n`entity2`.","ref":"Smee.Metadata.html"},{"type":"function","title":"Smee.Metadata.check_date!/1","doc":"Raises an exception if the metadata has expired (based on valid_until datetime), otherwise returns the metadata.\n\nIf no valid_until has been set (if it's nil) then the metadata will always be returned.","ref":"Smee.Metadata.html#check_date!/1"},{"type":"function","title":"Smee.Metadata.compress/1","doc":"Returns compressed metadata, containing gzipped XML. This greatly reduces the size of the metadata record.","ref":"Smee.Metadata.html#compress/1"},{"type":"function","title":"Smee.Metadata.compressed?/1","doc":"Returns true if the XML data in an metadata struct has been compressed","ref":"Smee.Metadata.html#compressed?/1"},{"type":"function","title":"Smee.Metadata.count/1","doc":"Returns the number of entities in the metadata file","ref":"Smee.Metadata.html#count/1"},{"type":"function","title":"Smee.Metadata.decompress/1","doc":"Returns decompressed metadata, with plain-text XML data. This makes the struct much larger.","ref":"Smee.Metadata.html#decompress/1"},{"type":"function","title":"Smee.Metadata.derive/2","doc":"Returns a new metadata struct based on the streamed entities passed as the first parameter.\n\nYou can set or override various parts of the struct by passing options:\n\n* url - the original location of the metadata\n* uri - a URI that identifies the metadata (Name)\n* downloaded_at - A DateTime to record when the record was downloaded\n* modified_at - A DateTime to record when the record was updated *upstream*\n* valid_until - A DateTime to indicate when an entity expires\n* priority - An integer between 0 and 9 to show priority\n* trustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.\n* etag - a string to use as an etag (unique content identifier).\n* cert_url - location of a certificate to use for signature verification\n* cert_fingerprint - fingerprint of the certificate to use for certificate verification\n* label - a description for the metadata","ref":"Smee.Metadata.html#derive/2"},{"type":"function","title":"Smee.Metadata.entities/1","doc":"Returns all entities in the metadata as a list of entity structs.\n\nThis can produce very large lists very slowly. The `stream_entities2` function is much better.","ref":"Smee.Metadata.html#entities/1"},{"type":"function","title":"Smee.Metadata.entity/2","doc":"Returns the specified entity from the metadata in an :ok/:error struct","ref":"Smee.Metadata.html#entity/2"},{"type":"function","title":"Smee.Metadata.entity!/2","doc":"Returns the specified entity from the metadata or raises an exception if not found","ref":"Smee.Metadata.html#entity!/2"},{"type":"function","title":"Smee.Metadata.entity_ids/1","doc":"Returns a list of all entity IDs in the metadata","ref":"Smee.Metadata.html#entity_ids/1"},{"type":"function","title":"Smee.Metadata.expired?/1","doc":"Returns true if the metadata has expired (based on valid_until datetime)\n\nIf no valid_until has been set (if it's nil) then false will be returned","ref":"Smee.Metadata.html#expired?/1"},{"type":"function","title":"Smee.Metadata.filename/1","doc":"Returns a suggested filename for the metadata.","ref":"Smee.Metadata.html#filename/1"},{"type":"function","title":"Smee.Metadata.filename/2","doc":"Returns a suggested filename for the metadata in the specified format.\n\nTwo formats can be specified: :sha1 and :uri","ref":"Smee.Metadata.html#filename/2"},{"type":"function","title":"Smee.Metadata.new/2","doc":"Returns a new metadata struct based on the XML data passed as the first parameter.\n\nYou can set or override various parts of the struct by passing options:\n\n* url - the original location of the metadata\n* uri - a URI that identifies the metadata (Name)\n* downloaded_at - A DateTime to record when the record was downloaded\n* modified_at - A DateTime to record when the record was updated *upstream*\n* valid_until - A DateTime to indicate when an entity expires\n* priority - An integer between 0 and 9 to show priority\n* trustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.\n* etag - a string to use as an etag (unique content identifier).\n* cert_url - location of a certificate to use for signature verification\n* cert_fingerprint - fingerprint of the certificate to use for certificate verification\n* label - a description for the metadata\n\nIn most cases it is better to use `Smee.Source` and then `Smee.Fetch` to generate a metadata struct.","ref":"Smee.Metadata.html#new/2"},{"type":"function","title":"Smee.Metadata.random_entity/1","doc":"Returns one randomly selected entity from the metadata","ref":"Smee.Metadata.html#random_entity/1"},{"type":"function","title":"Smee.Metadata.stream_entities/2","doc":"Returns a stream of all entities in the metadata.","ref":"Smee.Metadata.html#stream_entities/2"},{"type":"function","title":"Smee.Metadata.tag/2","doc":"Tags a metadata record with one or more tags, replacing existing tags.\n\nTags are arbitrary classifiers, initially inherited from sources","ref":"Smee.Metadata.html#tag/2"},{"type":"function","title":"Smee.Metadata.tags/1","doc":"Returns the tags of the metadata struct, a list of binary strings\n\nTags are arbitrary strings, which may be initially inherited from source records, and will be passed on to entities.","ref":"Smee.Metadata.html#tags/1"},{"type":"function","title":"Smee.Metadata.update/1","doc":"Resyncs the internal state of a %Metadata{} struct\n\nIf changes have been made using `Metadata.update/2` then this will not be needed - it's there for when the struct\n  has been changed directly","ref":"Smee.Metadata.html#update/1"},{"type":"function","title":"Smee.Metadata.update/2","doc":"Returns an updates %Metadata{} struct with new XML, refreshing various parts of the struct correctly.\n\nThis should be the only way updated Metadata structs are produced - the raw struct should not be changed directly.","ref":"Smee.Metadata.html#update/2"},{"type":"function","title":"Smee.Metadata.validate!/1","doc":"Raises an exception if the metadata has invalid XML, otherwise returns the metadata.","ref":"Smee.Metadata.html#validate!/1"},{"type":"function","title":"Smee.Metadata.xdoc/1","doc":"Returns a parsed Erlang `xmerl` structure representing the metadata XML, for use with `xmerl`, `SweetXML` and other\n  tools.\n\nUsing this is *not recommended* as it will create a very, very large `xmerl` structure. The `Smee.Transform` and `Smee.Extract`\n  modules may be more efficient for working with large metadata files, and the best approach is to stream and work with\n`Smee.Entity` records using `Smee.Metadata.stream_entities/2`\n\nUnlike the similar function for `Entity` it is not possible to cache this in the struct, so it will\nbe regenerated every time.","ref":"Smee.Metadata.html#xdoc/1"},{"type":"function","title":"Smee.Metadata.xml/1","doc":"Returns the XML for the metadata, unchanged, and decompressed.\n\nThe XML is returned as a binary string - it may be **very** large, and larger than the struct it comes from.","ref":"Smee.Metadata.html#xml/1"},{"type":"function","title":"Smee.Metadata.xml_processed/2","doc":"Returns the XML for the metadata, decompressed, after a processing stage.\n\nAvailable processing options:\n  * `:default` and `:none` - Nothing is changed, so it will be the same output as `Smee.Metadata.xml/1`\n  * `:strip` - XML has comments removed, signature removed, and XML declaration removed.\n\nThe XML is returned as a binary string - it may be **very** large.","ref":"Smee.Metadata.html#xml_processed/2"},{"type":"type","title":"Smee.Metadata.t/0","doc":"","ref":"Smee.Metadata.html#t:t/0"},{"type":"module","title":"Smee.Publish","doc":"Publish exports streams or lists of entity structs into various formats.\n\nAt present the output formats are SAML XML (individual and collections) and simple index text files. Formats can be\noutput as binary strings or streamed. Streamed output can be useful for web services, allowing gradual downloads generated\non-the-fly with no need to render a very large document in advance.\n\nOptions:\n* `valid_until` - pass a DateTime to set the validUntil attribute for the entity metadata. Alternatively, an integer can be\n  passed to request a validity of n days, or \"default\" and \"auto\" to use the default validity period.","ref":"Smee.Publish.html"},{"type":"function","title":"Smee.Publish.estimate_index_size/2","doc":"Returns the estimated size of a streamed index file without generating it in advance.","ref":"Smee.Publish.html#estimate_index_size/2"},{"type":"function","title":"Smee.Publish.estimate_xml_size/2","doc":"Returns the estimated size of a streamed SAML metadata XML file without generating it in advance.","ref":"Smee.Publish.html#estimate_xml_size/2"},{"type":"function","title":"Smee.Publish.index/2","doc":"Returns an index text document","ref":"Smee.Publish.html#index/2"},{"type":"function","title":"Smee.Publish.index_stream/2","doc":"Returns a streamed index file, a plain text list of entity IDs.","ref":"Smee.Publish.html#index_stream/2"},{"type":"function","title":"Smee.Publish.xml/2","doc":"Returns a SAML metadata XML file, potentially very large.","ref":"Smee.Publish.html#xml/2"},{"type":"function","title":"Smee.Publish.xml_stream/2","doc":"Returns a streamed SAML metadata XML file","ref":"Smee.Publish.html#xml_stream/2"},{"type":"module","title":"Smee.Security","doc":"`Smee.Security` uses XML signatures to provide anti-tampering and origin authentication features.\n\nMost metadata from national research and education networks is signed to add an extra layer of security for users:\n  altered metadata can create security and privacy vulnerabilities for organisations.\n\nAt the moment only signature verification on Metadata structs is supported.","ref":"Smee.Security.html"},{"type":"function","title":"Smee.Security.verify/1","doc":"Returns a metadata struct in an :ok/:error tuple, with `verified: true` set if verification passes.","ref":"Smee.Security.html#verify/1"},{"type":"function","title":"Smee.Security.verify!/1","doc":"Returns a metadata struct with `verified: true` set if verification passes, or raises an exception.","ref":"Smee.Security.html#verify!/1"},{"type":"function","title":"Smee.Security.verify?/1","doc":"Returns true or false depending on verification status of the passed metadata.\n\nAlready-verified metadata will not be reverified.","ref":"Smee.Security.html#verify?/1"},{"type":"module","title":"Smee.Sigils","doc":"`Smee.Sigils` provides a version of SweetXml's `~x` sigil for creating XPaths, optimised for working with SAML\n  metadata. All of Smee's known XML namespaces will be automatically loaded for you.\n\nFor more information please read [the official SweetXml documentation](https://hexdocs.pm/sweet_xml/SweetXml.html#module-the-x-sigil).\n\n `import Smee.Sigils` to use. If you are also importing SweetXml functions you should avoid loading both Smee and\n  SweetXml's versions of this sigil:\n\n```\n  import SweetXml, except: [sigil_x: 2]\n  import Smee.Sigils\n```\n\nThis sigil does not automatically precompile or cache, so it's advisable to use a module attribute to store the compiled\n  xpath if you can.","ref":"Smee.Sigils.html"},{"type":"function","title":"Smee.Sigils.sigil_x/2","doc":"A convenient replacement for","ref":"Smee.Sigils.html#sigil_x/2"},{"type":"module","title":"Smee.Source","doc":"Defines sources for metadata, which can then be `Fetch`ed and prodoce Metadata structs. Source structs are the\n  usual starting-place for Smee tasks.\n\n Source structs act as the configuration for metadata creation. They require a URL but various other options can be\n   set and their effect will propagate down to Entity records.","ref":"Smee.Source.html"},{"type":"function","title":"Smee.Source.check/2","doc":"Attempts to validate a source struct, and will return an :ok/:error tuple containing the Source if it passes checks.","ref":"Smee.Source.html#check/2"},{"type":"function","title":"Smee.Source.check!/2","doc":"Attempts to validate a source struct, and will return the Source if it passes checks, or raise an exception.","ref":"Smee.Source.html#check!/2"},{"type":"function","title":"Smee.Source.new/2","doc":"Creates a new source struct, describing where and how to find metadata.\n\nThe only essential parameter is a URL for the metadata. URLs can have `file:`, `http://`, or `https://` schemes.\nBare filesystem paths can also be passed and will be converted to `file:` URLs.\n\nMDQ service clients should be configured with the *base URL* of the service, not the `/entity/` endpoint, and a\n type of :mdq. Alternatively the `Smee.MDQ.source/2` function can be used as a shortcut.\n\nAvailable options include:\n\n  * type: determines the general type of metadata - :aggregate (the default), :single and :mdq\n  * cert_url: location of certificate to use when verifying signed metadata\n  * cert_fingerprint: SHA1 fingerprint of the signing certificate\n  * cache: should HTTP caching be enabled for downloads. true or false, defaults to true.\n  * redirects: maximum number of 302 redirects to follow\n  * retries: number of retries to attempt\n  * label: a relatively useless label for the metadata\n  * priority: integer between 0 and 9, used for comparing metadata\n  * trustiness: float between 0.0 and 0.9, for comparing metadata\n  * strict: defaults to false. If enabled some stricter checks are enabled\n\nMDQ sources are intended for use with the `Smee.MDQ` API but will also support normal fetch requests.","ref":"Smee.Source.html#new/2"},{"type":"function","title":"Smee.Source.tag/2","doc":"Tags a source with one or more tags, replacing existing tags.\n\nTags are arbitrary strings. They will be inherited by metadata and entities derived from this source.","ref":"Smee.Source.html#tag/2"},{"type":"function","title":"Smee.Source.tags/1","doc":"Returns the tags of the source struct, a list of binary strings\n\nTags are arbitrary classifiers","ref":"Smee.Source.html#tags/1"},{"type":"type","title":"Smee.Source.t/0","doc":"","ref":"Smee.Source.html#t:t/0"},{"type":"module","title":"Smee.Stats","doc":"A collection of simple statistics tools that use streams of entity structs (and maybe lists)","ref":"Smee.Stats.html"},{"type":"function","title":"Smee.Stats.count/1","doc":"Counts the number of entities in the stream and returns a value (runs and ends the stream)\n\nIs this a placeholder in an almost empty module? Absolutely.","ref":"Smee.Stats.html#count/1"},{"type":"module","title":"Smee.Sys","doc":"Contains utility functions for integrating Smee with larger applications, managing caches and working files, and so on.","ref":"Smee.Sys.html"},{"type":"function","title":"Smee.Sys.reset_cache/0","doc":"Removes **all** files from the Smee download cache.\n\nUnlike temporary files, download cache files will remain between runs of Smee applications, and the cache directory\n  can become much too large over time. It's best to run `reset_cache/1` on application startup and maybe schedule it\n to run every few weeks too.\n\n The number of cache files that have been deleted will be returned in an :ok tuple.","ref":"Smee.Sys.html#reset_cache/0"},{"type":"module","title":"Smee.Transform","doc":"Tools for manipulating metadata XML.\n\nThis module is intended to make common and relatively simple actions on potentially large XML files more efficiently than\nprocessing a metadata or entities using xmerl, and may rely on external tools such as `xsltproc`.\n\nThese functions are useful for pre-processing metadata XML before it's broken up into entity struct,\nand to speed-up the internal workings of other modules.","ref":"Smee.Transform.html"},{"type":"function","title":"Smee.Transform.decruft_idp/1","doc":"Strips various extraneous parts from the metadata and returns a new struct in an :ok/:error tuple","ref":"Smee.Transform.html#decruft_idp/1"},{"type":"function","title":"Smee.Transform.decruft_sp/1","doc":"Strips various extraneous parts from the metadata and returns a new struct in an :ok/:error tuple","ref":"Smee.Transform.html#decruft_sp/1"},{"type":"function","title":"Smee.Transform.transform/3","doc":"Applies an XSLT stylesheet to a metadata struct, returning a transformed metadata struct in an :ok/error tuple","ref":"Smee.Transform.html#transform/3"},{"type":"function","title":"Smee.Transform.valid_until/2","doc":"Returns a metadata struct with the validUntil date updated, in an :ok/:error struct","ref":"Smee.Transform.html#valid_until/2"},{"type":"function","title":"Smee.Transform.valid_until!/2","doc":"Returns a metadata struct with the validUntil date updated, or raises if an error occurs","ref":"Smee.Transform.html#valid_until!/2"},{"type":"extras","title":"Smee","doc":"# Smee\n\n`Smee` is a pragmatic library for handling SAML metadata with Elixir, Erlang or any other BEAM language.\n\nSmee started life as an sprawling mess of code used over five years in an internal application to download and process\nSAML metadata. It's now been extracted, restructured and made a **little** less messy, with the aim of reusing it in other \nprojects. Smee can be used within applications or in simpler scripts for processing metadata.\n\n[![Hex pm](http://img.shields.io/hexpm/v/smee.svg?style=flat)](https://hex.pm/packages/smee)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](http://hexdocs.pm/smee/)\n![Github Elixir CI](https://github.com/Digital-Identity-Labs/smee/workflows/Elixir%20CI/badge.svg)\n\n[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2FDigital-Identity-Labs%2Fsmee%2Fmain%2Fsmee_notebook.livemd)","ref":"readme.html"},{"type":"extras","title":"Features - Smee","doc":"* Download remote SAML metadata or load local files, with effective caching\n* Manage and compare metadata files and individual entity metadata \n* MDQ API (which can also emulate MDQ style lookups with aggregate files)\n* A focus on streaming with reliable and surprisingly low memory usage\n* Filter entity streams by various criteria\n* Validate XML signatures, automatically download and confirm signing certificates\n* Transform metadata using XSLT, or extract data\n* Access XML using Erlang's Xmerl library (sweetened by SweetXML)\n* Recombine entity streams into aggregates or other data formats\n* Can be used with applications or in simple .exs scripts\n\nThe top level `Smee` module contains simplified, top level functions better suited to simpler scripts. Other modules in\nSmee contain more tools for handling SAML metadata, such as:\n\n* `Smee.Source` - define sources of metadata\n* `Smee.Metadata` - functions for handling metadata aggregates\n* `Smee.Entity` - individual SAML entity definitions\n* `Smee.Extract` - processing metadata to extract information\n* `Smee.Fetch` - downloading metadata sources\n* `Smee.MDQ` - functions for MDQ clients (and emulating MDQ clients)\n* `Smee.Filter` - filtering streams of entity records\n* `Smee.Transform` - processing and editing entity XML\n* `Smee.Publish` - Formatting and outputting metadata in various formats\n* `Smee.Stats` - Simple stats for entity streams\n* `Smee.Sigils` - An ~x sigil for XPaths optimized for use with SAML","ref":"readme.html#features"},{"type":"extras","title":"Extensions and Extras - Smee","doc":"* [SmeeFeds](https://github.com/Digital-Identity-Labs/smee_feds) - a federation management extension for use in\n  research, testing and development. SmeeFeds has a built-in list of many education and research federations.","ref":"readme.html#extensions-and-extras"},{"type":"extras","title":"Examples - Smee","doc":"#","ref":"readme.html#examples"},{"type":"extras","title":"Getting entity details from an MDQ service - Smee","doc":"```elixir\nalias Smee.MDQ\n\ncern_idp = \n  MDQ.source(\"http://mdq.ukfederation.org.uk/\")\n  |> MDQ.lookup!(\"https://cern.ch/login\")\n\n```\n\n#","ref":"readme.html#getting-entity-details-from-an-mdq-service"},{"type":"extras","title":"Downloading and streaming a metadata aggregate to output a list of all IdP entityIDs - Smee","doc":"```elixir\nalias Smee.{Source, Fetch, Filter, Metadata}\n\n\"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\"\n|> Source.new()\n|> Fetch.remote!()\n|> Metadata.stream_entities()\n|> Filter.idp()\n|> Stream.map(fn e -> e.uri end)\n|> Enum.to_list()\n\n```\n\n#","ref":"readme.html#downloading-and-streaming-a-metadata-aggregate-to-output-a-list-of-all-idp-entityids"},{"type":"extras","title":"Create an aggregate that only contains SPs with SIRTFI, that were registered over the previous 12 months - Smee","doc":"```elixir\nalias Smee.{Source, Fetch, Filter, Metadata, Publish}\n\n\"http://metadata.ukfederation.org.uk/ukfederation-metadata.xml\"\n|> Source.new()\n|> Fetch.remote!()\n|> Metadata.stream_entities()\n|> Filter.days(365)\n|> Filter.sp()\n|> Filter.assurance(\"https://refeds.org/sirtfi\")\n|> Publish.xml()\n\n```\n#","ref":"readme.html#create-an-aggregate-that-only-contains-sps-with-sirtfi-that-were-registered-over-the-previous-12-months"},{"type":"extras","title":"Munge XML with XSLT - Smee","doc":"```elixir\nalias Smee.{Source, Fetch, Transform, Metadata}","ref":"readme.html#munge-xml-with-xslt"},{"type":"extras","title":"Using a builtin function - Smee","doc":"new_xml = Source.new(\"adfs.xml\")\n|> Fetch.local!()\n|> Transform.decruft_sp!()\n|> Metadata.xml()\n\nFile.save!(\"adfs_new.xml\", new_xml)","ref":"readme.html#using-a-builtin-function"},{"type":"extras","title":"Or any XSLT stylesheet - Smee","doc":"{:ok, updated_metadata} = Transform.transform(metadata, stylesheet, [exampleParam: \"example value\"])\n\n```","ref":"readme.html#or-any-xslt-stylesheet"},{"type":"extras","title":"Installation - Smee","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `Smee` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:Smee, \"~> 0.4.1\"}\n  ]\nend\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Backend tools - Smee","doc":"**Please note:** Smee does not do all processing itself using Elixir - it sometimes cheats (OK, it often cheats) by sending data to\nexternal programs for processing. At the moment it requires the following commandline utilities:\n\n* `xmlsec1`\n* `xmllint`\n* `xsltproc`\n\nSmee now includes a Mix task to install the default backend software - just run:\n\n`mix deps.smee`\n\n(Currently only tested on Macs with Homebrew, but it *should* also work on Debian, Ubuntu, Red Hat and Alpine Linux)\n\nA future version of Smee will support alternative sets of backends.\n\n#","ref":"readme.html#backend-tools"},{"type":"extras","title":"Rambo and Rust - Smee","doc":"Smee uses an Elixir library called [Rambo](https://hex.pm/packages/rambo) to run external utilities like xsltproc. Rambo \nrelies on a small compiled shim that is provided pre-compiled for various architectures or automatically built during installation.\n\n- If your system does not match any of the supplied binary shims, you will need to have a rust compiler installed so Rambo\n  can build the shim itself. \n- Version 0.3.4 of Rambo has an additional problem: it does not ship with a precompiled binary for M1 Macs, *and it also does\n  not automatically build one*. Smee works around that with an explicit compile step in `mix.exs`, but it prevent Smee from\n  being used in Elixir .exs scripts on M1 Macs unless you copy the compiled shim from Smee into your script's build directory.\n  Hopefully this will be fixed in later versions of Rambo. \n- If you are using a modern M1/M2/M3 Mac you may need to explicitly include Rambo as a dependency in your project.","ref":"readme.html#rambo-and-rust"},{"type":"extras","title":"Uses - Smee","doc":"Metadata Database: Smee was extracted from an application that downloads, processes and stores SAML metadata in a database for access \nwith an API. The original code had become rather messy and was very difficult to test - hopefully Smee will be an \nimprovement and will be used to replace the original code. Smee doesn't currently handle digestion and storage.\n\nMDQ Service: We've made a streaming MDQ service using Smee that should soon be available \n\nReports: We've generated reports for clients using Smee","ref":"readme.html#uses"},{"type":"extras","title":"Possible Alternatives - Smee","doc":"These aren't quite the same but are mature, widely used projects that are a lot more stable than Smee. In many cases you \nshould definitely use these instead of Smee.\n\n* [SAML Library by Tom Scavo](https://github.com/trscavo/saml-library/) A collection of Bash and XSLT scripts \n* [PyFF](https://github.com/IdentityPython/pyFF) A metadata pipeline and server using Python and YAML\n* [Shibboleth Metadata Aggregator](https://shibboleth.atlassian.net/wiki/spaces/MA1/overview) Widely used Java metadata pipeline","ref":"readme.html#possible-alternatives"},{"type":"extras","title":"References - Smee","doc":"#","ref":"readme.html#references"},{"type":"extras","title":"SAML Metadata - Smee","doc":"* [Wikipedia's summary](https://en.wikipedia.org/wiki/SAML_2.0)\n\n#","ref":"readme.html#saml-metadata"},{"type":"extras","title":"MDQ - Smee","doc":"* [UK Federation Explains MDQ](https://www.ukfederation.org.uk/content/Documents/MDQ)\n\n#","ref":"readme.html#mdq"},{"type":"extras","title":"Unusual types of SAML metadata - Smee","doc":"* [Oauth/OIDC](https://shibboleth.atlassian.net/wiki/spaces/SC/pages/1912406916/OAuthRPMetadataProfile)\n* [ADFS does it's own thing](https://shibboleth.atlassian.net/wiki/spaces/SC/pages/1836417024/ADFSMetadataProfile)\n* [CAS!](https://shibboleth.atlassian.net/wiki/spaces/SC/pages/1879344655/CASMetadataProfile)","ref":"readme.html#unusual-types-of-saml-metadata"},{"type":"extras","title":"Contributors and Sources - Smee","doc":"Various Github repositories have been plundered in a search for XSLT stylesheets, in particular:\n\n* [SAML Library by Tom Scavo](https://github.com/trscavo/saml-library/)\n* [SWAMID Federation](https://swamid.se/swamid-metadata.git/tree/xslt)\n* [UK Access Management Federation's metadata processing pipeline](https://github.com/ukf/ukf-meta)\n\nThe use of the Apache2 license in the above projects led to the Apache2 license for Smee.\n\nOther sources and inspirations:\n\n* [The XSLT package for Elixir](https://github.com/johnhamelink/xslt)\n\nSmee temporarily includes a copy of [EasySSL](https://github.com/CaliDog/EasySSL) (MIT Licensed) that was required to\nfix an Erlang 26 compatibility issue. This will revert to a normal dependency when an update is released.","ref":"readme.html#contributors-and-sources"},{"type":"extras","title":"Documentation - Smee","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Smee","doc":"You can request new features by creating an [issue](https://github.com/Digital-Identity-Labs/smee/issues),\nor submit a [pull request](https://github.com/Digital-Identity-Labs/smee/pulls) with your contribution.\n\nIf you are comfortable working with Python but Smee's Elixir code is unfamiliar then this blog post may help: \n[Elixir For Humans Who Know Python](https://hibox.live/elixir-for-humans-who-know-python)","ref":"readme.html#contributing"},{"type":"extras","title":"Copyright and License - Smee","doc":"Copyright (c) 2023, 2024 Digital Identity Ltd, UK\n\nSmee is Apache 2.0 licensed.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Disclaimer - Smee","doc":"Smee is not endorsed by The Shibboleth Foundation or any of the NREN's that may have had their XSLT borrowed.\nThe API will definitely change considerably in the first few releases after 0.1.0 - it is not stable!","ref":"readme.html#disclaimer"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}